#include "msw.h"
#include <avr/io.h>
#include <avr/interrupt.h>

#include "usbdrv.h"

uint32_t counter = 0;
uint32_t time = 0; 
//int8_t pind_memory;
uint8_t small_counter = 0;
char beep_on = 1;

uint32_t temp_counter = 0;
uint32_t temp_time = 0;
char text[8];

inline void init_timer()
{
	cbi(TCCR1A, WGM11);
	cbi(TCCR1A, WGM10);
	#if defined (__AVR_ATtiny4313__)
	TCCR1B = (0 << CS12)|(0 << CS11)|(1 << CS10)|(0<<WGM13)|(0<<WGM12);
	#endif
	#if defined(__AVR_ATmega32__)
	TCCR1B = (1 << CS12)|(0 << CS11)|(1 << CS10)|(0<<WGM13)|(0<<WGM12);
	#endif
/*presacling
  CS12 CS11 CS10
    0    0    1  = 1:1
    0    1    0  = 1:8
    0    1    1  = 1:64
    1    0    0  = 1:256
    1    0    1  = 1:1024
	
*/	
	OCR1A = F_CPU/1000;
	//sbi(TIMSK, TOIE1);//for ATmega32 and ATtiny4313
	sbi(TIMSK, OCIE1A);
}

void init_timer0_counter()
{
	cbi(TCCR0A, WGM01);
	cbi(TCCR0A, WGM00);
	TCCR0B = (1 << CS02)|(1 << CS01)|(0 << CS00)|(0<<WGM02);//on falling edge
	TCNT0 = 0;
	sbi(TIMSK, TOIE0);//enable overflow interruption	
}

/*int16_t get_timer()
{
	/*int8_t low = TCNT1L;
	int8_t high = TCNT1H;
	return 256*high + low;*/
	/*return TCNT1;
}*/

inline void set_timer(int16_t n)
{
	/*int8_t low = n & 0xff;
	int8_t high = n/256;
	TCNT1H = high;
	TCNT1L = low;*/
	TCNT1 = n;
}

inline void beep(int t)
{
	sbi(PORTB, BUZZER);
	_delay_ms(t);
	cbi(PORTB, BUZZER);
}

int beep_if_change()
{
	static uint8_t temp_small = 0;
	if (temp_small != TCNT0) 
	{
		if (beep_on) beep(5);
		temp_small = TCNT0;
	}		
} 

ISR(TIMER1_COMPA_vect)
{	
	set_timer(0);
	time++;
}

ISR(INT1_vect)
{
	counter++;
	if (beep_on) beep(5);
}

ISR(TIMER0_OVF_vect)
{
	counter += 256;
}

int _init()
{
	sbi(DDRD, 0); //Ustawiamy nieuzywane piny jako wyjscia i stan wysoki aby uodpornic na zaklocenia!
	sbi(DDRD, 1);
	sbi(DDRD, 6);
	
	sbi(PORTD, 0);
	sbi(PORTD, 1);
	sbi(PORTD, 6);
	
//	sbi(DDRB, GM_SUPPLY);//init as out
//	cbi(PORTB, GM_SUPPLY);//400V on (1 - off, 0 - on)
	
	sbi(DDRB, BUZZER);//init as out
	cbi(PORTB, BUZZER);//BUZZER off

	sbi(DDRB, LCD_BL);//init as out
	sbi(PORTB, LCD_BL);//LCD BL on

	beep_on = 1;
		
	init_LCD();
	clr_LCD();
	goto_I_line_LCD();
	print_string("Radioactive@Home");
	goto_II_line_LCD();
	print_string(" Please wait...");
	
	//cbi(PORTB, GM_SUPPLY); //400V On

	//_delay_ms(1000);

	cbi(DDRD, PULSE);//init PULSE as in
	sbi(PORTD, PULSE);

	init_timer();
	set_timer(0);
	init_timer0_counter();
	sei();

    //beep(1000);//Long beep
	//_delay_ms(2000);
	//pind_memory = PIND;
}

inline void msw_itoa(uint32_t i, char *s)
{
	char *c = s;
	*s = 0;
	do{
		*(++s) = i%10 + '0';
		i = i/ 10;
	}while(i>0);

	char d;
	while (s > c){
		d = *s;
		*s = *c;
		*c = d;
		c++;
		s--;
	}

}

inline void div100(char *s)
{
	char *c = s;
	while(*c) c++;
	if (c == s) return;
	if (s + 1 == c){
		*c = *s;
		*s = '0';
		c++;
	}
	*c = *(c-1);
	*(c-1) = *(c-2);
	*(c-2) = '.';
	*(++c) = 0;
	if (*s == '.'){
		*(c+1) = 0;
		while(c!=s){
			*c = *(c-1);
			c--;
		}
		*c = '0';
	}
	
}

uint8_t length(char *s)
	{
		uint8_t len = 0;
		while(*(s++)) len++;
		return len;
	} 

void msw_main()
{
	beep_if_change();
	
	if (time - temp_time >= 35040) //35040
	{
		goto_II_line_LCD();
		print_string("                ");

		beep_if_change();
		
		small_counter = TCNT0;

		if (counter + (uint32_t) small_counter - temp_counter <= 999999)
		{
			msw_itoa(counter + (uint32_t) small_counter - temp_counter, text);

			beep_if_change();
		
			div100(text);
		
			beep_if_change();
		
			goto_LCD(72 - length(text)); //align center

			beep_if_change();
		
			print_string(text);
			print_string(" uSv/h");
		}
        else
		{
		goto_LCD(66);
		print_string("out of range");
		}		

		beep_if_change();
		
		small_counter = TCNT0;
		temp_counter = counter + (uint32_t) small_counter;
		temp_time = time;
		
		beep_if_change();
	}
	
}
